#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>


int main() {
    printf("TI-85 Rom Fixer by Davr http://blog.davr.org/\n");
    printf("Ported from the app at http://www.aub.dk/~dines/dump.shtml\n");
    printf("\n");
    printf("Enter TI-85 Rom version:\n"
            "2 :  2.0\n"
            "3 :  3.0A\n"
            "4 :  4.0\n"
            "5 :  5.0\n"
            "6 :  6.0\n"
            "8 :  8.0\n"
            "9 :  9.0\n"
            "0 : 10.0\n"
            ">");
    int romtype = getchar();

    romtype -= '0';
    if(romtype == 0)
        romtype = 10;

    if(romtype < 2 || romtype == 7 || romtype > 10) {
        printf("ERROR unkown rom version. Press ENTER to exit.");
        getchar();
        exit(1);
    }

    unsigned char bytes[] = {
        0,
        0,
        0x2A,
        0x57,
        0x8E,
        0x5E,
        0x3D,
        0,
        0x97,
        0x91,
        0x35,
        0};

    struct stat st;
    stat("ti85.rom", &st);
    if(st.st_size != 131072) {
        perror("Wrong size! Ti-85 rom should be exactly 131072 bytes!\nPress Enter to exit");
        getchar();
        exit(1);
    }

    FILE *fp = fopen("ti85.rom", "rb");
    if(!fp) {
        perror("opening file. Press ENTER to exit.");
        getchar();
        exit(1);
    }

    char *filebuf = (char *)malloc(st.st_size);
    if(!filebuf) {
        fclose(fp);
        perror("mallocing. Press ENTER to exit.");
        getchar();
        exit(1);
    }

    int bytesin=0;
    while(!feof(fp) && bytesin < st.st_size) {
        bytesin += fread(&filebuf[bytesin], 1, st.st_size-bytesin, fp);
    }
    fclose(fp);
    if(bytesin < st.st_size) {
        perror("reading all file. Press ENTER to exit.");
        getchar();
        exit(1);
    }
    fp = fopen("ti85_rom.bak", "wb");
    if(!fp) {
        perror("opening backup. Press ENTER to exit.");
        getchar();
        exit(1);
    }
    int bytesout = fwrite(filebuf, 1, st.st_size, fp);
    fclose(fp);
    if(bytesout < st.st_size) {
        perror("saving backup. Press ENTER to exit.");
        getchar();
        exit(1);
    }

    for(int i=1; i<=8; i++) {
        if(i < 8)
            filebuf[i*16384 - 1] = 0xFF;
        else
            filebuf[i*16384 - 1] = bytes[romtype];
    }

    fp = fopen("ti85.rom", "wb");
    if(!fp) {
        perror("opening output. Press ENTER to exit.");
        getchar();
        exit(1);
    }

    bytesout = fwrite(filebuf, 1, st.st_size, fp);
    printf("wrote %d bytes.\n", bytesout);
    fclose(fp);
    if(bytesout < st.st_size) {
        perror("saving output. Press ENTER to exit.");
        getchar();
        exit(1);
    }

    printf("Patched successfully!\nFixed version: ti85.rom\nBackup of old rom: ti85_rom.bak\nPress ENTER to exit.\n");
    getchar();
    return 0;

}
